#!/usr/bin/python
# Copyright (C) 2019 Jelmer Vernooij
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Tests for readme parsing."""

import os
import platform
from unittest import TestCase, TestSuite

from upstream_ontologist.readme import (
    description_from_readme_md,
    description_from_readme_rst,
    description_from_readme_plain,
)


class ReadmeTestCase(TestCase):

    def __init__(self, path):
        super().__init__()
        self.path = path

    def setUp(self):
        super().setUp()
        self.maxDiff = None

    def runTest(self):
        readme_md = None
        readme_rst = None
        readme_plain = None
        description = None
        for entry in os.scandir(self.path):
            if entry.name.endswith('~'):
                continue
            base, ext = os.path.splitext(entry.name)
            if entry.name == 'description':
                with open(entry.path) as f:
                    description = f.read()
            elif base == "README":
                if ext == '.md':
                    with open(entry.path) as f:
                        readme_md = f.read()
                elif ext == '.rst':
                    with open(entry.path) as f:
                        readme_rst = f.read()
                elif ext == '':
                    with open(entry.path) as f:
                        readme_plain = f.read()
                else:
                    raise NotImplementedError(ext)
            else:
                raise NotImplementedError(ext)

        if readme_md is not None:
            try:
                import markdown  # noqa: F401
            except ModuleNotFoundError:
                self.skipTest(
                    'Skipping README.md tests, markdown not available')
            actual_description, unused_md = description_from_readme_md(
                readme_md)
            self.assertEqual(actual_description, description)

        if readme_rst is not None:
            if platform.python_implementation() == "PyPy":
                self.skipTest('Skipping README.rst tests on pypy')
            try:
                import docutils  # noqa: F401
            except ModuleNotFoundError:
                self.skipTest(
                    'Skipping README.rst tests, docutils not available')
            actual_description, unused_rst = description_from_readme_rst(
                readme_rst)
            self.assertEqual(actual_description, description)

        if readme_plain is not None:
            actual_description, unused_rst = description_from_readme_plain(
                readme_plain)
            self.assertEqual(actual_description, description)


def test_suite():
    suite = TestSuite()
    for entry in os.scandir(os.path.join(os.path.dirname(__file__), 'readme_data')):
        suite.addTest(ReadmeTestCase(entry.path))
    return suite



#!/usr/bin/python
# Copyright (C) 2022 Jelmer Vernooij
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Tests for guess."""

from unittest import TestCase

from upstream_ontologist import UpstreamDatum, Person
from upstream_ontologist.guess import (
    metadata_from_itp_bug_body
)


class MetadataFromItpBugBody(TestCase):

    def test_simple(self):
        self.assertEqual([
            UpstreamDatum('Name', 'setuptools-gettext', 'confident'),
            UpstreamDatum('X-Version', '0.0.1', 'possible'),
            UpstreamDatum(
                'X-Author', [Person.from_string(
                    'Breezy Team <breezy-core@googlegroups.com>')], 'confident'),
            UpstreamDatum('Homepage', 'https://github.com/jelmer/setuptools-gettext', 'confident'),
            UpstreamDatum('X-License', 'GPL', 'confident'),
            UpstreamDatum('X-Summary', 'Compile .po files into .mo files', 'confident'),
            UpstreamDatum('X-Description', """\
This extension for setuptools compiles gettext .po files
found in the source directory into .mo files and installs them.
""", 'likely')
        ], list(metadata_from_itp_bug_body("""\
Package: wnpp
Severity: wishlist
Owner: Jelmer Vernooij <jelmer@debian.org>
X-Debbugs-Cc: debian-devel@lists.debian.org

* Package name    : setuptools-gettext
  Version         : 0.0.1
  Upstream Author : Breezy Team <breezy-core@googlegroups.com>
* URL             : https://github.com/jelmer/setuptools-gettext
* License         : GPL
  Programming Lang: Python
  Description     : Compile .po files into .mo files

This extension for setuptools compiles gettext .po files
found in the source directory into .mo files and installs them.

""")))



#!/usr/bin/python3
# Copyright (C) 2021 Jelmer Vernooij <jelmer@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

from typing import List
import urllib.error
from urllib.request import Request, urlopen
from urllib.parse import urljoin

import logging

from . import UpstreamDatum, USER_AGENT

logger = logging.getLogger(__name__)


def guess_from_homepage(url: str):
    req = Request(url, headers={'User-Agent': USER_AGENT})
    try:
        f = urlopen(req)
    except urllib.error.HTTPError as e:
        logger.warning(
            'unable to access homepage %r: %s', url, e)
        return
    except urllib.error.URLError as e:
        logger.warning(
            'unable to access homepage %r: %s', url, e)
        return
    except ConnectionResetError as e:
        logging.warning(
            'unable to access homepage %r: %s', url, e)
        return
    for entry in _guess_from_page(f.read(), url):
        entry.origin = url
        yield entry


def _guess_from_page(text: bytes, basehref: str):
    try:
        from bs4 import BeautifulSoup, FeatureNotFound
    except ModuleNotFoundError:
        logger.debug('BeautifulSoup not available, not parsing homepage')
        return
    try:
        soup = BeautifulSoup(text, 'lxml')
    except FeatureNotFound:
        logger.debug('lxml not available, not parsing README.md')
        return
    yield from _guess_from_soup(soup, basehref)


def _guess_from_soup(soup, basehref):
    for a in soup.findAll('a'):
        href = a.get('href')
        labels: List[str] = [a.get('aria-label'), a.text]
        for label in filter(None, labels):
            if label.lower() in ('github', 'git', 'repository', 'github repository'):
                yield UpstreamDatum('Repository', urljoin(basehref, href), certainty='possible')
            if label.lower() in ('github bug tracking', 'bug tracker'):
                yield UpstreamDatum('Bug-Database', urljoin(basehref, href), certainty='possible')



#!/usr/bin/python3
# Copyright (C) 2018 Jelmer Vernooij <jelmer@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Functions for working with upstream metadata."""

import logging
import os
import sys

from . import (
    version_string,
    UpstreamDatum,
    Person,
)
from .guess import (
    guess_upstream_metadata,
    guess_upstream_info,
)


def main(argv=None):
    import argparse
    import ruamel.yaml

    parser = argparse.ArgumentParser(sys.argv[0])
    parser.add_argument("path", default=".", nargs="?")
    parser.add_argument(
        "--trust",
        action="store_true",
        help="Whether to allow running code from the package.",
    )
    parser.add_argument(
        "--disable-net-access",
        help="Do not probe external services.",
        action="store_true",
        default=False,
    )
    parser.add_argument(
        "--check",
        action="store_true",
        help="Check guessed metadata against external sources.",
    )
    parser.add_argument(
        "--scan",
        action="store_true",
        help="Scan for metadata rather than printing results.",
    )
    parser.add_argument(
        "--from-homepage",
        type=str,
        help="Scan specified homepage rather than current directory")
    parser.add_argument(
        "--consult-external-directory",
        action="store_true",
        help="Pull in external (not maintained by upstream) directory data",
    )
    parser.add_argument(
        "--version", action="version", version="%(prog)s " + version_string
    )
    parser.add_argument('--verbose', action='store_true')
    args = parser.parse_args(argv)

    if args.verbose:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)

    if not os.path.isdir(args.path):
        sys.stderr.write("%s is not a directory\n" % args.path)
        return 1

    if args.from_homepage:
        from .homepage import guess_from_homepage
        for entry in guess_from_homepage(args.from_homepage):
            if isinstance(entry, UpstreamDatum):
                print(
                    "%s: %r - certainty %s (from %s)"
                    % (entry.field, entry.value, entry.certainty, entry.origin)
                )
    elif args.scan:
        for entry in guess_upstream_info(args.path, args.trust):
            if isinstance(entry, UpstreamDatum):
                print(
                    "%s: %r - certainty %s (from %s)"
                    % (entry.field, entry.value, entry.certainty, entry.origin)
                )
            else:
                raise TypeError(entry)
    else:
        metadata = guess_upstream_metadata(
            args.path,
            args.trust,
            not args.disable_net_access,
            consult_external_directory=args.consult_external_directory,
            check=args.check,
        )

        yaml = ruamel.yaml.YAML()
        ruamel.yaml.scalarstring.walk_tree(metadata)
        yaml.register_class(Person)
        # Prefer .buffer so we can write e.g. surrogates
        yaml.dump(metadata, getattr(sys.stdout, 'buffer', sys.stdout))
        return 0


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/python3
# Copyright (C) 2022 Jelmer Vernooij <jelmer@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

from typing import Dict, Any


def codemeta_file_from_upstream_info(upstream_info):
    ret: Dict[str, Any] = {
        "@context": "https://doi.org/10.5063/schema/codemeta-2.0",
        "@type": "SoftwareSourceCode",
    }
    if "Name" in upstream_info:
        ret["name"] = upstream_info["Name"]
    if "X-Version" in upstream_info:
        ret["version"] = upstream_info["X-Version"]
    if "Repository" in upstream_info:
        ret["codeRepository"] = upstream_info["Repository"]
    if "Bug-Database" in upstream_info:
        ret["issueTracker"] = upstream_info["Bug-Database"]
    # TODO(jelmer): Support setting contIntegration
    if "X-License" in upstream_info:
        ret["license"] = "https://spdx.org/licenses/%s" % upstream_info["X-License"]
    if "X-Description" in upstream_info:
        ret["description"] = upstream_info["X-Description"]
    # TODO(jelmer): Support keywords
    # TODO(jelmer): Support funder
    # TODO(jelmer): Support funding
    # TODO(jelmer): Support creation date
    # TODO(jelmer): Support first release date
    # TODO(jelmer): Support unique identifier
    # TODO(jelmer): Support runtime platform
    # TODO(jelmer): Support other software requirements
    # TODO(jelmer): Support operating system
    # TODO(jelmer): Support development status
    # TODO(jelmer): Support reference publication
    # TODO(jelmer): Support part of
    # TODO(jelmer): Support Author
    for link_field in ["Documentation", "Homepage"]:
        if link_field in upstream_info:
            ret.setdefault("relatedLink", []).append(upstream_info[link_field])
    if "X-Download" in upstream_info:
        ret["downloadUrl"] = upstream_info["X-Download"]
    return ret


def main(argv=None):
    from .guess import get_upstream_info
    import argparse
    import sys
    import json

    if argv is None:
        argv = sys.argv

    parser = argparse.ArgumentParser(argv)
    parser.add_argument("path", default=".", nargs="?")

    parser.add_argument(
        "--trust",
        action="store_true",
        help="Whether to allow running code from the package.",
    )
    parser.add_argument(
        "--disable-net-access",
        help="Do not probe external services.",
        action="store_true",
        default=False,
    )
    parser.add_argument(
        "--check",
        action="store_true",
        help="Check guessed metadata against external sources.",
    )

    args = parser.parse_args()

    upstream_info = get_upstream_info(
        args.path, trust_package=args.trust,
        net_access=not args.disable_net_access,
        check=args.check)

    codemeta = codemeta_file_from_upstream_info(upstream_info)

    json.dump(codemeta, sys.stdout, indent=4)


if __name__ == '__main__':
    import sys
    sys.exit(main(sys.argv))
